/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */


#ifndef __JFTPGW_CMDS_H__
#define __JFTPGW_CMDS_H__

struct conn_info_st {
	struct log_cmd_st* lcs;
	struct clientinfo* clntinfo;
};

struct cmdhandlerstruct {
	char* cmd;
	int (*func)(const char*, struct proxyinfo*);
};

/* it doesn't matter when we call an analyze function if there are several.
 * All have to say OK to a command */
#define CMD_HANDLERTYPE_PREEXEC			 50
#define CMD_HANDLERTYPE_IMPLEMENT		150
#define CMD_HANDLERTYPE_POSTEXEC		250

#define CMD_HANDLERTYPE_RANGE_IMPLEMENT_MIN	100
#define CMD_HANDLERTYPE_RANGE_IMPLEMENT_MAX	199

#define CMD_SERIALIZE_REMOVE_NONE		0
#define CMD_SERIALIZE_REMOVE_ONE		1
#define CMD_SERIALIZE_REMOVE_ALL		2

typedef int(*cmd_hookfunc_t)(const char*, const char**,
				struct proxyinfo*, struct messagebuffer*);
struct cmd_handler_t {
	int priority;
	const char* cmd_name;
	const char* description;
	int ncmd_args_min;
	int ncmd_args_max;
	int serial_id;
	cmd_hookfunc_t func;
	const char* expect;
	cmd_hookfunc_t err_reset;
};

/* functions from ftp_cmds.c */

struct cmd_handler_t*
cmd_get_next_handler(const char*, int, struct cmd_handler_t*);
int cmd_register(int priority,
		 int serialize,
		 const char* cmd_name,
		 const char* description,
		 int ncmd_args_min,
		 int ncmd_args_max,
		 int serial_id,
		 cmd_hookfunc_t hookfunc,
		 const char* expect,
		 cmd_hookfunc_t unexpected);
void cmd_set_initial_expect(const char* expected);
void cmd_handlers_reset();

int transfer_initiate(struct proxyinfo*, struct messagebuffer*);
int transfer_negotiate(struct proxyinfo*, struct messagebuffer*);
int transfer_transmit(struct proxyinfo*);
int transfer_cleanup(struct proxyinfo*);

int std_cmds_init(void);

#endif
