/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __JPROXYGW_CONFIG_HEADER_H__
#define __JPROXYGW_CONFIG_HEADER_H__

#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "types.h"


#define TAG_NUMBER         9
#define TAG_UNKNOWN        (-1)

#define TAG_GLOBAL         1
#define TAG_GLOBAL_STR     "global"
#define TAG_FROM           2
#define TAG_FROM_STR       "from"
#define TAG_TO             4
#define TAG_TO_STR         "to"
#define TAG_USER           8
#define TAG_USER_STR       "user"
#define TAG_PORT           16
#define TAG_PORT_STR       "port"
#define TAG_FORWARDED      32
#define TAG_FORWARDED_STR  "forwarded"
#define TAG_TIME           64
#define TAG_TIME_STR       "time"
#define TAG_SERVERTYPE     128
#define TAG_SERVERTYPE_STR "servertype"
#define TAG_PROXYIP        256
#define TAG_PROXYIP_STR    "proxyip"
#define TAG_PROXYPORT      512
#define TAG_PROXYPORT_STR  "proxyport"

#define TAG_OPENING        0
#define TAG_CLOSING        1

#define TAG_ALL (TAG_FROM | TAG_TO | TAG_GLOBAL | TAG_USER | TAG_PORT | TAG_FORWARDED | TAG_TIME | TAG_SERVERTYPE | TAG_PROXYIP | TAG_PROXYPORT)
#define TAG_ALL_NOT_FORWARDED  (TAG_ALL & (~TAG_FORWARDED))
#define TAG_STARTUP            (TAG_GLOBAL | TAG_SERVERTYPE)
#define TAG_CONNECTED          (TAG_STARTUP | TAG_FROM | TAG_PROXYIP | TAG_PROXYPORT | TAG_TIME)

#define SERVERTYPE_STANDALONE		0
#define SERVERTYPE_INETD		1

#ifndef WHITESPACES
#define WHITESPACES		" \t"
#endif

#define BROADCAST		"255.255.255.255"

struct tag_options_t {
	char* list_str;
	char* list_exclude_str;
};

struct timestruct {
	struct ilist_t* days;
	int start_day;
	int start_hour;
	int start_minute;
	int end_day;
	int end_hour;
	int end_minute;
	struct timestruct* next;
};

struct section_t {
	int                                tag_name;
	unsigned int                       id;
	struct hostlist_t*                 hosts;
	struct hostlist_t*                 hosts_exclude;
	struct slist_t*                    users;
	struct slist_t*                    users_exclude;
	struct slist_t*                    forwarded;
	struct slist_t*                    forwarded_exclude;
	struct portrangestruct*            ports;
	struct portrangestruct*            ports_exclude;
	struct timestruct*                 time;
	struct timestruct*                 time_exclude;
	int                                servertype;
	struct option_prio_t*              options_prio;
	struct section_t*                  nested;
	struct section_t*                  next;
	long int connection_counter;
	long int limit;
};


#define OPTION_EXACT_MATCH     1
#define READ_UP_TO_WS          1
#define READ_FULL_LINE         2
#define EM                     OPTION_EXACT_MATCH
#define WSP                    READ_UP_TO_WS
#define FL                     READ_FULL_LINE

/* Table of configuration options (identifiers, validity, defaultvalues) */
struct config_directives_list {
	char* name;
	int tag_type;
	char* defaultvalue;
	int match_type;
	int read_limit;
};

struct config_namespace_directives_list {
	char* namespace_name;
	struct config_directives_list* directives_list;
};

struct config_configuration_list {
	struct config_namespace_directives_list* namespace;
	struct config_configuration_list* next;
};


#define TERM ((char*) 0)
#define TRUEFALSE "on", "off", "yes", "no", "true", "false", "1", "0"
#define MAXCONFOPTVALS			23	/* main.facility */
struct config_option_values_list {
	char* name;
	char* values[ MAXCONFOPTVALS ];
};

struct config_namespace_options_list {
	char* namespace_name;
	struct config_option_values_list* options_list;
};

struct config_options_list {
	struct config_namespace_options_list* namespace;
	struct config_options_list* next;
};


/* exported functions from config.c */
int read_config(const char*, struct proxyinfo*);

/* hook registration */
typedef int(*config_hookfunc_t)(void);
int config_register_hook(int, config_hookfunc_t);

int config_shrink_config(unsigned long int, /* from ip */
			unsigned long int,  /* dest ip */
			const char*,        /* dest hostname */
			unsigned int,       /* dest port */
			const char*,        /* dest user */
			unsigned long int,  /* forwarded ip */
			const char*,        /* forwarded hostname */
			unsigned int,       /* forwarded destinationport */
			const char*,        /* forwarded user */
			time_t,             /* specific time */
			unsigned long int,  /* proxy ip */
			unsigned int,       /* proxy port */
			int,                /* servertype */
			int,                /* ready_to_serve */
			struct hostent_list**, int);

int config_counter_decrease(unsigned long int,            /* from_ip */
			    unsigned long int,            /* proxy_ip */
			    unsigned int,                 /* proxy_port */
			    time_t,                       /* specific_time */
			    int,                          /* servertype */
			    struct hostent_list**);       /* hostcache */

int config_counter_increase(unsigned long int,            /* from_ip */
			    unsigned long int,            /* proxy_ip */
			    unsigned int,                 /* proxy_port */
			    time_t,                       /* specific_time */
			    int,                          /* servertype */
			    struct hostent_list**);       /* hostcache */
int config_check_limit_violation(void);
void config_counter_add_connected(struct connliststruct*, int, struct hostent_list**);
const char* config_get_option(const char* key);
struct slist_t* config_get_option_array(const char* key);

char** quotstrtok_arr(const char*, const char*);
char* quotstrtok(const char*, const char*, int*);
char* quotstrtok_prepend(const char*, const char*, const char*, int*);

struct slist_t* config_split_line(const char* line, const char* pattern);
struct slist_t* slist_reverse(struct slist_t* sl);
int slist_case_contains(const struct slist_t*, const char*);
void slist_destroy(struct slist_t* sl);
char* slist_pop(struct slist_t* sl);
int slist_count(const struct slist_t* haystack);
void config_destroy_portrange(struct portrangestruct* plist);
int config_get_ioption(const char* key, int err_return);
long config_get_loption(const char* key, long err_return);
unsigned long int config_get_addroption(const char* key, unsigned long int);
int config_get_bool(const char* key, int err_return);
float config_get_foption(const char* key, float err_return);
unsigned long int config_get_size(const char* key, unsigned long int err);
/* these modify the config_options_list */
void config_option_list_delete(const char* key);
void config_option_list_add(const char* key, const char* value);
int config_compare_option(const char* key, const char* compare);
void config_delete_config();
void config_delete_backup();
void config_delete_master();
void config_create_backup();
int config_activate_backup();
void config_destroy_sectionconfig();
void config_deinit();

const char* hostent_get_name(struct hostent_list** h, unsigned long int ip);
unsigned long int hostent_get_ip(struct hostent_list** h, const char* name);

long conv_char2long(const char*, long);

struct portrangestruct* config_parse_portranges(const char* line);
unsigned int config_count_portrange(const struct portrangestruct* prs);
struct portrangestruct* config_port2portrange(unsigned int port);

void config_configuration_list_add(struct config_namespace_directives_list*);
void config_options_list_add(struct config_namespace_options_list*);

const char* gethostentip(const char* iplist);
char* trim(char *const);

/* external */
char* chrooted_path(const char* path);
FILE* open_file(const char*);

#endif
